#include "Ballet256256.h"  

#include "Ballet256256_coreE.h"
#include "Ballet256256_coreD.h"

static void DecBallet256_256B_1W_Round(unsigned char *out,unsigned char *pt,  unsigned char *rk,int round);
static void EncBallet256_256B_1W_Round(unsigned char *out,unsigned char *pt,  unsigned char *rk,int round);

static  void BalletGenRK_256_256(u8i * Rk, u8i * K,int dirt); 
static  void Ballet256256EncDataS_Round(u8i *out, u8i *in, u8i *K, int len,int round);



/*
Ballet 256-256  interface 
*/

int Crypt_Enc_Block(unsigned char *input,int in_len, unsigned char  *output, int *out_len, unsigned char *key , int keylen)
{

	u64i rk[RoundBallet256256 * 4];
	int i;
	if(in_len<1)
		return -1;
	if(keylen!=32)
		return -2;  
	BalletGenRK_256_256_ENC((u8i *)rk, key);
	for (i = 0; i < in_len; i += 32)
		Ballet256256EncDataS(output + i, input + i, (u8i *)rk);
	*out_len= in_len;
	return 0;
}

int Crypt_Dec_Block(unsigned char *input,int in_len, unsigned char *output,int *out_len, unsigned char *key, int keylen)
{
	__m256i rkm[RoundBallet256256 * 2];
	u8i TText[256];
	int leftbytes;
	if (in_len < 1)
		return -1;
	if (keylen != 32)
		return -2;
	BalletGenRK_256_256M_DEC(rkm, key);
	 
	leftbytes = in_len;
	while (leftbytes >= 256)
	{
		Ballet256256DecDataM(output, input, (u8i *)rkm);
		output += 256;
		input += 256;
		leftbytes -= 256;
	}
	if (leftbytes)
	{
		memcpy(TText, input, leftbytes);
		Ballet256256DecDataM(TText, TText, (u8i *)rkm);
		memcpy(output, TText, leftbytes);
	} 
	*out_len= in_len;
	return 0;
}
//IV ȫ 0

int Crypt_Enc_Block_CBC(unsigned char *input, int in_len, unsigned char  *output, int *out_len, unsigned char *key, int keylen)
{
	u64i rk[RoundBallet256256 * 4];
	__m256i IV;
	int i;
	if (in_len < 1)
		return -1;
	if (in_len %32)
		return -2;
	if (keylen != 32)
		return -3;
	BalletGenRK_256_256_ENC((u8i *)rk, key);
	Ballet256256EncDataS(output, input, (u8i *)rk);
	for (i = 32; i < in_len; i += 32)
	{
		IV = _mm256_xor_si256(_mm256_loadu_si256((__m256i *)(input + i)), _mm256_loadu_si256((__m256i *)(output + i - 32)));
		Ballet256256EncDataS(output + i, (u8i *)(&IV), (u8i *)rk);
	}
	*out_len = in_len;
	return 0;
}

int Crypt_Dec_Block_CBC(unsigned char *input, int in_len, unsigned char  *output, int *out_len, unsigned char *key, int keylen)
{
	u8i _C_ALIGN16(TText[256]);
	__m256i rkm[RoundBallet256256 * 2];
	__m256i mIV = _mm256_set1_epi64x(0);
	int leftbytes;
	if (in_len < 1)
		return -1;
	if (in_len % 32)
		return -2;
	if (keylen != 32)
		return -3;
	BalletGenRK_256_256M_DEC(rkm, key);
	leftbytes = in_len;
	while (leftbytes >= 256)
	{
		Ballet256256DecDataM_CBC(output, input, (u8i *)rkm, &mIV);
		output += 256;
		input += 256;
		leftbytes -= 256;
	}
	if (leftbytes)
	{
		memcpy(TText, input, leftbytes);
		Ballet256256DecDataM_CBC(TText, TText, (u8i *)rkm, &mIV);
		memcpy(output, TText, leftbytes);
	}
	*out_len = in_len;
	return 0;
}
//ֻּӽ
//ӽԿ洢ռ䲻ͬ 
void Key_Schedule(unsigned char *Seedkey, int KeyLen, unsigned   char Direction, unsigned char *Subkey)
{
	if (Direction == Dirt_ENC)
		BalletGenRK_256_256_ENC(Subkey, Seedkey);
	else
		BalletGenRK_256_256M_DEC((__m256i *)Subkey, Seedkey);
}

int Crypt_Enc_Block_Round(unsigned char *input,int in_len,  unsigned char *output, int *out_len, unsigned char *key, int keylen, int CryptRound)
{
	if(in_len<1)
		return -1;
	if(keylen!=32)
		return -2;
	if((CryptRound> RoundBallet256256) || (CryptRound<0))
		return -3; 
	Ballet256256EncDataS_Round(output, input, key, in_len,CryptRound);
	*out_len= in_len;
	return 0;
}

/*
end of ballet 256-256 interface
*/

static void BalletGenRK_256_256(u8i * Rk, u8i * K,int dirt)
{
	u64i i;
	u64i  k00,k01,k10,k11;
	u64i *p=(u64i *)Rk;
	k01= *((u64i *)K);
	k00= *((u64i *)(K+8));
	k11= *((u64i *)(K+16));
	k10= *((u64i *)(K+24));

	k00 =SWAP64(k00);
	k01 =SWAP64(k01);
	k10 =SWAP64(k10);
	k11 =SWAP64(k11);  
	if(dirt==Dirt_ENC)
	{
		for(  i=0;i<RoundBallet256256;i+=2 )  
		{
			*p= k00;
			*(p+1)= k01;  
			k00^= ((k10<<3)|(k11>>61) )^((k10<<5)|(k11>>59))^i;
			k01^= ((k11<<3)|(k10>>61) )^((k11<<5)|(k10>>59)); 
			*(p+2)= k10;
			*(p+3)= k11;  
			k10^= ((k00<<3)|(k01>>61)  )^((k00<<5)|(k01>>59) )^(i+1);
			k11^= ( (k01<<3)|(k00>>61) )^((k01<<5)|(k00>>59) ); 
			p+=4; 
		} 

	} 
	else
	{
		p+=(RoundBallet256256*2-1);
		for(  i=0;i<RoundBallet256256;i+=2 )  
		{
			*(p-1)= k00;
			*(p)= k01; 
			k00^= ((k10<<3)|(k11>>61) )^((k10<<5)|(k11>>59))^i;
			k01^= ((k11<<3)|(k10>>61) )^((k11<<5)|(k10>>59)); 
			*(p-3)= k10;
			*(p-2)= k11; 
			k10^= ((k00<<3)|(k01>>61)  )^((k00<<5)|(k01>>59) )^(i+1);
			k11^= ( (k01<<3)|(k00>>61) )^((k01<<5)|(k00>>59) ); 
			p-=4; 
		} 
	} 
} 



static void EncBallet256_256B_1W_Round(unsigned char *out,unsigned char *pt,  unsigned char *rk,int round)
{
	int i;
	u64i t0; 
	u64i *trk = (u64i *)rk; 
	u64i a = *((u64i *)pt);
	u64i b = *((u64i *)(pt+8));
	u64i c = *((u64i *)(pt+16));	
	u64i d = *((u64i *)(pt+24));	

	a =SWAP64(a);b =SWAP64(b);
	c =SWAP64(c); d =SWAP64(d); 

	for(  i=0;i<round;i++)
	{     
		if(i%2==0)
		{ 
			t0= b^c;   	
			b^=  (*(trk+1) );
			c^= (*trk); 
			d= ROTL64(d,15);  
			a=ROTL64(a,6); 
			a+=ROTL64(t0,9); 
			d+=ROTL64(t0,14); 
		}
		else
		{
			t0= a^d;  
			b=ROTL64(b,6); 
			b+=ROTL64(t0,9);  
			d^= (*(trk));
			a^= (*(trk+1));   
			c= ROTL64(c,15);  
			c+=ROTL64(t0,14);  
		}
		trk+=2; 

	} 
	a =SWAP64(a);b =SWAP64(b);
	c =SWAP64(c); d =SWAP64(d);
	if(round%2)
	{
		*((u64i *)out)=a;
		*((u64i *)(out+8))=b;
		*((u64i *)(out+16))=c;	
		*((u64i *)(out+24))=d;
	}
	else
	{  
		*((u64i *)out)=b;
		*((u64i *)(out+8))=a;
		*((u64i *)(out+16))=d;	
		*((u64i *)(out+24))=c;
	}		
}


static void DecBallet256_256B_1W_Round(unsigned char *out,unsigned char *pt,  unsigned char *rk,int round)
{
	int i;
	u64i t0; 
	u64i *trk = (u64i *)rk; 
	u64i a = *((u64i *)pt);
	u64i b = *((u64i *)(pt+8));
	u64i c = *((u64i *)(pt+16));	
	u64i d = *((u64i *)(pt+24));	

	a =SWAP64(a);b =SWAP64(b);
	c =SWAP64(c); d =SWAP64(d);  
	for(  i=0;i<round;i++)
	{         	
		if(i%2==0)
		{
			b^=  (*(trk+1) );
			c^= (*trk);  
			t0=b^c; 
			a-=ROTL64(t0,9);
			d-=ROTL64(t0,14);
			a= ROTR64(a, 6);
			d= ROTR64(d, 15); 
		}
		else
		{
			a^=  (*(trk+1));
			d^= (*(trk));  
			t0= a^d;
			b-=ROTL64(t0,9);
			c-=ROTL64(t0,14);	
			b= ROTR64(b, 6);
			c= ROTR64(c, 15); 
		}
		trk+=2;

	}

	a =SWAP64(a);b =SWAP64(b);
	c =SWAP64(c); d =SWAP64(d);	
	if(round%2==0)
	{
		*((u64i *)out)=b;
		*((u64i *)(out+8))=a;
		*((u64i *)(out+16))=d;	
		*((u64i *)(out+24))=c;	
	}
	else
	{
		*((u64i *)out)=a;
		*((u64i *)(out+8))=b;
		*((u64i *)(out+16))=c;	
		*((u64i *)(out+24))=d;		
	}
}

#define BytesperEncS 32

 
 


void Ballet256256EncDataS_Round(u8i *out, u8i *in, u8i *K, int len,int round)
{ 

	u64i rk[RoundBallet256256*4];
	int i;
	BalletGenRK_256_256((u8i *)rk,K,Dirt_ENC);
	for(  i=0;i<len;i+=BytesperEncS)  
		EncBallet256_256B_1W_Round(out+i, in+i, (u8i *)rk,round);   
#ifdef OutputInnerPars
	printf("round %02d : ", round-1);
		for(  j=0;j<32;j++)
		{
			printf("%02X", out[j]);
		   if((j+1)%8==0)
			   printf(" ");
		}
		printf(" rk%02d: ",round-1);
			printf("%016I64X ", rk[round*2-1]); 
			printf("%016I64X ", rk[round*2-2]); 
		printf("\n"); 
#endif
}

